##------------------------------------------------------------##
##                Review of Linear Models                     ##
##                       John Fox                             ##
##                      York SPIDA                            ##
##                         2012                               ##
##------------------------------------------------------------##

# multiple regression

library(car)  # load car package

?Duncan  # help page for data set

some(Duncan)  # random few rows

View(Duncan)  # browse data set

scatterplotMatrix(~ prestige + income + education, data=Duncan, id.n=3)

scatterplot(income ~ education, data=Duncan, id.method="identify")

##Multiple Regression Model - Section 2.1, slide 3##
duncan.mod <- lm(prestige ~ income + education, data=Duncan)
summary(duncan.mod)

##Confidence Intervals - Section 2.2, slide 9##
confint(duncan.mod)

    # incremental F-test - Section 2.2.3, slide 14##
		    
duncan.mod.inc <- update(duncan.mod, . ~ . - education) # remove education

##Individual Slope coefficients - Section 2.2.1, slide 9##
anova(duncan.mod.inc, duncan.mod)  # test if education coefficient = 0

anova(duncan.mod) # sequential ("Type-I") tests

Anova(duncan.mod) # partial ("Type-II") tests (equivalent to t-tests)

    # linear hypothesis - Section 2.2.4, slide 16##
    
(L <- matrix(c(0, 1, -1), 1, 3)) # hypothesis matrix
linearHypothesis(duncan.mod, L, 0)

linearHypothesis(duncan.mod, "education = income") # equivalent

# dummy regression - Section 4##

    # dichotomous factor - Section 4.1, slide 35 ##

scatterplot(repwt ~ weight | sex, data=Davis, smooth=FALSE, id.n=1)

head(Davis, 12)
  
Davis[12, c(2, 3)] <- Davis[12, c(3, 2)] # correct data
head(Davis, 12)

Davis$sex
class(Davis$sex)
contrasts(Davis$sex)  # dummy regressors for sex, slide 40 ##

options("contrasts")

mod.davis.1 <- lm(weight ~ repwt + sex, data=Davis)  # additive model - Section 4.1 - slide 41##
summary(mod.davis.1)

mod.davis.2 <- lm(weight ~ repwt*sex, data=Davis)  # model with interaction, slide 51 and 55##
summary(mod.davis.2)

anova(mod.davis.1, mod.davis.2)  # test of interaction - Section 4.3.1, slide 58##

Anova(mod.davis.2)  # type II tests, obeying marginality - Section 4.4, slide 59##

linearHypothesis(mod.davis.2,     # slopes AND intercepts
  c("sexM = 0", "repwt:sexM = 0")) # the same for M and F
  
confidenceEllipse(mod.davis.2, which=3:4) # 95% confidence ellipse - Section 3.2##
abline(h=0, v=0, lty=2)
                                                        
    # polytomous factor - Section 4.2, slide 46##

View(Prestige)
Prestige <- na.omit(Prestige) # remove cases with missing data (be careful!)

    # treatment of factors in R

Prestige$type
class(Prestige$type)
contrasts(Prestige$type)  # dummy regressors

Prestige$type <- factor(Prestige$type, levels=c("bc", "wc", "prof"))  # change order of levels
contrasts(Prestige$type)

    # additive dummy-regression model - Section 4.1.2, slide 43##

mod.prestige.1 <- lm(prestige ~ income + education + type, data=Prestige)
summary(mod.prestige.1)

mod.prestige.0 <- update(mod.prestige.1, . ~ . - type)
anova(mod.prestige.0, mod.prestige.1)  # incremental F-test for type

linearHypothesis(mod.prestige.1, c("typewc = 0", "typeprof = 0"))
Anova(mod.prestige.1)

    # dummy regression with interactions - Section 4.3##

mod.prestige.2 <- lm(prestige ~ income + education +
                   type + income:type + education:type, data=Prestige)

mod.prestige.2 <- lm(prestige ~ income*type + education*type, data=Prestige) # equivalent

mod.prestige.2 <- lm(prestige ~ type*(income + education), data=Prestige) # equivalent
summary(mod.prestige.2)

Anova(mod.prestige.2)  # type-II tests

    # "effect" plots

library(effects)  # for "effect" displays
plot(allEffects(mod.prestige.2), ask=FALSE)
plot(allEffects(mod.prestige.2), multiline=TRUE, ask=FALSE)

# regression diagnostics -  Section 5##

    # unusual data - Section 5.1, slide 63 ##
    
        # preliminary example

remove(Davis)  # get rid of fixed data set
        
scatterplot(repwt ~ weight|sex, smooth=FALSE, id.n=1, 
  data=Davis)

scatterplot(weight ~ repwt|sex, smooth=FALSE, id.n=1,
  data=Davis)

davis.1 <- lm(repwt ~ weight*sex, data=Davis)
summary(davis.1)

davis.2 <- lm(weight ~ repwt*sex, data=Davis)
summary(davis.2)

##Measuring Influence - Section 5.1.3, slide 75##

influencePlot(davis.1, id.n=1)
influenceIndexPlot(davis.1, id.n=2)

qqPlot(davis.1, simulate=TRUE, id.n=2)  # QQ plot for stud. res. - Detecting Outliers - Section 5.1.2, slide 70##

        # Duncan's regression
		
## Multiple Regression Model - Section 2.1, slide 3##        
duncan <- lm(prestige ~ income + education, data=Duncan)
summary(duncan)

avPlots(duncan, ask=FALSE, id.method="identify")  # added-variable plots - Section 5.1.4, slide 77##

which.names(c("minister", "conductor"), Duncan)

##Measuring Influence - Section 5.1.3, slide 75##
influencePlot(duncan, id.n=3)

##Detecting Outliers - Section 5.1.2, slide 70##
qqPlot(duncan, simulate=TRUE, id.n=2, reps=1000)

outlierTest(duncan)

duncan.2 <- update(duncan, subset= -c(6,16)) # removing ministers & conductors
summary(duncan.2)
linearHypothesis(duncan.2, "income = education") ## Linear Hypotheses - Section 2.2.4##

    # non-normality -  Section 5.2, slide 83##
    
SLID <- read.table(
"http://socserv.socsci.mcmaster.ca/jfox/Books/Applied-Regression-2E/datasets/SLID-Ontario.txt", 
    header=TRUE)

slid0 <- lm(compositeHourlyWages ~ sex + age + yearsEducation, data=SLID)
summary(slid0)

qqPlot(slid0, line="none")

plot(density(rstudent(slid0)))

slid1 <- lm(log2(compositeHourlyWages) ~ sex + age + yearsEducation, data=SLID)
summary(slid1)

qqPlot(slid1, line="none")

plot(density(rstudent(slid1)))

boxplot(rstudent(slid1))

    # non-constant error variance - Section 5.3, slide 90##
    
plot(fitted(slid0), rstudent(slid0), col="gray")
abline(h=0, lty=2)
lines(lowess(fitted(slid0), rstudent(slid0)))

spreadLevelPlot(slid0)

plot(fitted(slid1), rstudent(slid1), col="gray")
abline(h=0, lty=2)
lines(lowess(fitted(slid1), rstudent(slid1)))

spreadLevelPlot(slid1)

    # nonlinearity - Section 5.4, slide 93##
		
		##Transformations - Section 5.2.1, slide 86##	
    
crPlots(slid1, ask=FALSE)

slid2 <- lm(log2(compositeHourlyWages) ~ sex + poly(age, degree=2) 
    + I(yearsEducation^2), data=SLID) # orthogonal polynomial in age
summary(slid2)
Anova(slid2)

lm(log2(compositeHourlyWages) ~ sex + poly(age, degree=2, raw=TRUE) 
    + I(yearsEducation^2), data=SLID) # "raw" quadratic

lm(log2(compositeHourlyWages) ~ sex + age + I(age^2) + I(yearsEducation^2), 
    data=SLID) # equivalent
    
crPlots(slid2, ask=FALSE)

plot(allEffects(slid2), ask=FALSE)  # effect plots

plot(allEffects(slid2, transformation=list(link=log2, inverse=function(x) 2^x)), 
    ylab="composite hourly wages", ask=FALSE)  # plot on original dollar scale

