##------------------------------------------------------------##
##              Introduction to Data in R                     ##
##                       John Fox                             ##
##                      York SPIDA                            ##
##                         2012                               ##
##------------------------------------------------------------##


# Data in R

    # reading data

        # entering data at the keyboard

(x <- c(1, 2, 3, 4)) # numeric data
(names <- c("John", "Sandy", 'Mary')) # character data
(v <- c(TRUE, FALSE)) # logical data


cooperation <- scan()
    49 64 37 52 68 54
    61 79 64 29
    27 58 52 41 30 40 39
    44 34 44

cooperation
    
        # patterned data
    
rep(5, 3)
rep(c(1, 2, 3), 2)
rep(1:3, 3:1)

(condition <- rep(c("public", "anonymous"), c(10, 10)))
(sex <- rep(rep(c("male", "female"), c(5, 5)), 2))

(Guyer <- data.frame(cooperation, condition, sex))


        # reading data from a file into a data frame
        
Prestige <- read.table("Prestige.txt", header=TRUE) # location in file system will vary
Prestige

Prestige <- read.table(file.choose(), header=TRUE) # alternative

        # reading data via the clipboard (e.g., from Excel)
        
Duncan <- read.table("clipboard", header=TRUE)
Duncan

remove(Duncan)

    
    # importing data from another package.
        
library(foreign)
Nations <- read.spss(file.choose(), to.data.frame=TRUE) 
View(Nations)

names(Nations) <- tolower(names(Nations))
rownames(Nations) <- Nations$nation
Nations$nation <- NULL
head(Nations)

remove(Nations)

    # accessing data in a package

library(car)
data(Duncan) # not necessary (if package supports "lazy loading" of data)
Duncan
remove(Duncan)

Duncan # from car via the lazy-loading mechanism

        # from a package that's not loaded:
        
data(Animals, package="MASS")
Animals

objects()
remove(Animals, names, v, x)

    # writing data to a file
    
write.table(Duncan, "c:/temp/Duncan.txt")

    # working with data frames

        # The search path

search()
prestige

Duncan[, "prestige"]

attach(Duncan)
prestige
search()

attach(Prestige)
search()
prestige    # prestige in Prestige shadows prestige in Duncan


Duncan[, "prestige"]  # still there!

detach(Prestige)
search()
prestige  # now from Duncan


mean(prestige)
mean(prestige, trim=0.1)

mean <- function(x){
    warning("the mean function in the base package is shadowed")
    sum(x)/length(x)
}

mean(prestige)
mean(prestige, trim=0.1)

remove(mean)
mean(prestige, trim=.1)

mean <- mean(prestige)  # variable named "mean" --  no problem!
mean
mean(prestige)

remove(mean)
detach(Duncan)

        # avoiding attach()
        
mean(Duncan$prestige) # explicit indexing

(lm(prestige ~ income + education, data=Duncan)) # using the data argument

with(Duncan, mean(prestige))  # using with()
with(Duncan, lm(prestige ~ income + education)) # better to use data argument

    # missing data

head(Freedman, 10)  # first 10 rows
tail(Freedman)  # last 6 rows
some(Freedman)  # 10 randomly sampled rows

Freedman$density
median(Freedman$density)
median(Freedman$density, na.rm=TRUE)

with(Freedman, {
    plot(density, crime) # NAs removed
    identify(density, crime, row.names(Freedman)) 
})

log(c(1, 10, NA, 100), base=10) # NAs propogated

with(Freedman, plot(log(density, base=10), crime))

lm(crime ~ log(density, base=10), data=Freedman) # NAs handled by na.action
getOption("na.action")

abline(lm(crime ~ log(density, base=10), data=Freedman), lty="dashed")

good <- with(Freedman, complete.cases(crime, density))
head(good, 20)  # first 20 values
with(Freedman,  # NAs handled by indexing:
    lines(lowess(log(density[good], base=10), crime[good], f=1.0)))

Freedman.good <- na.omit(Freedman) # filtering NAs
head(Freedman.good)  # first 6 rows
dim(Freedman.good)   # number of rows and columns
        
        # testing for NAs
        
NA == c(1, 2, NA, 4) # wrong!
is.na(c(1, 2, NA, 4))
sum(is.na(Freedman)) # count of NAs

objects()
remove(good, Freedman.good)

    # numeric variables, character variables, and factors

condition
is.character(condition)

condition <- as.factor(condition)
condition

remove(cooperation, condition, sex)

Guyer$condition
is.character(Guyer$condition)
is.factor(Guyer$condition)

summary(Guyer)

    # modifying data

Guyer$perc.coop <- 100*Guyer$cooperation/120 # assign to data frame
head(Guyer)  # first 6 rows

Guyer$cooperation <- with(Guyer, log(perc.coop/(100 - perc.coop))) # replace
head(Guyer)

